# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import sys
from typing import Any, AsyncIterable, Callable, Dict, IO, Optional, Type, TypeVar, Union, overload
import urllib.parse

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ...operations._operations import (
    build_variable_values_create_or_update_at_management_group_request,
    build_variable_values_create_or_update_request,
    build_variable_values_delete_at_management_group_request,
    build_variable_values_delete_request,
    build_variable_values_get_at_management_group_request,
    build_variable_values_get_request,
    build_variable_values_list_for_management_group_request,
    build_variable_values_list_request,
    build_variables_create_or_update_at_management_group_request,
    build_variables_create_or_update_request,
    build_variables_delete_at_management_group_request,
    build_variables_delete_request,
    build_variables_get_at_management_group_request,
    build_variables_get_request,
    build_variables_list_for_management_group_request,
    build_variables_list_request,
)

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class VariablesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.resource.policy.v2022_08_01_preview.aio.PolicyClient`'s
        :attr:`variables` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")
        self._api_version = input_args.pop(0) if input_args else kwargs.pop("api_version")

    @distributed_trace_async
    async def delete(self, variable_name: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """Deletes a variable.

        This operation deletes a variable, given its name and the subscription it was created in. The
        scope of a variable is the part of its ID preceding
        '/providers/Microsoft.Authorization/variables/{variableName}'.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_variables_delete_request(
            variable_name=variable_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    async def create_or_update(
        self, variable_name: str, parameters: _models.Variable, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.Variable:
        """Creates or updates a variable.

        This operation creates or updates a variable with the given subscription and name. Policy
        variables can only be used by a policy definition at the scope they are created or below.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param parameters: Parameters for the variable. Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update(
        self, variable_name: str, parameters: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.Variable:
        """Creates or updates a variable.

        This operation creates or updates a variable with the given subscription and name. Policy
        variables can only be used by a policy definition at the scope they are created or below.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param parameters: Parameters for the variable. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update(
        self, variable_name: str, parameters: Union[_models.Variable, IO[bytes]], **kwargs: Any
    ) -> _models.Variable:
        """Creates or updates a variable.

        This operation creates or updates a variable with the given subscription and name. Policy
        variables can only be used by a policy definition at the scope they are created or below.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param parameters: Parameters for the variable. Is either a Variable type or a IO[bytes] type.
         Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable or IO[bytes]
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Variable] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "Variable")

        _request = build_variables_create_or_update_request(
            variable_name=variable_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Variable", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def get(self, variable_name: str, **kwargs: Any) -> _models.Variable:
        """Retrieves a variable.

        This operation retrieves a single variable, given its name and the subscription it was created
        at.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.Variable] = kwargs.pop("cls", None)

        _request = build_variables_get_request(
            variable_name=variable_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Variable", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def delete_at_management_group(  # pylint: disable=inconsistent-return-statements
        self, management_group_id: str, variable_name: str, **kwargs: Any
    ) -> None:
        """Deletes a variable.

        This operation deletes a variable, given its name and the management group it was created in.
        The scope of a variable is the part of its ID preceding
        '/providers/Microsoft.Authorization/variables/{variableName}'.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_variables_delete_at_management_group_request(
            management_group_id=management_group_id,
            variable_name=variable_name,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    async def create_or_update_at_management_group(
        self,
        management_group_id: str,
        variable_name: str,
        parameters: _models.Variable,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Variable:
        """Creates or updates a variable.

        This operation creates or updates a variable with the given  management group and name. Policy
        variables can only be used by a policy definition at the scope they are created or below.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param parameters: Parameters for the variable. Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update_at_management_group(
        self,
        management_group_id: str,
        variable_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.Variable:
        """Creates or updates a variable.

        This operation creates or updates a variable with the given  management group and name. Policy
        variables can only be used by a policy definition at the scope they are created or below.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param parameters: Parameters for the variable. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update_at_management_group(
        self,
        management_group_id: str,
        variable_name: str,
        parameters: Union[_models.Variable, IO[bytes]],
        **kwargs: Any
    ) -> _models.Variable:
        """Creates or updates a variable.

        This operation creates or updates a variable with the given  management group and name. Policy
        variables can only be used by a policy definition at the scope they are created or below.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param parameters: Parameters for the variable. Is either a Variable type or a IO[bytes] type.
         Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable or IO[bytes]
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.Variable] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "Variable")

        _request = build_variables_create_or_update_at_management_group_request(
            management_group_id=management_group_id,
            variable_name=variable_name,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Variable", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def get_at_management_group(
        self, management_group_id: str, variable_name: str, **kwargs: Any
    ) -> _models.Variable:
        """Retrieves a variable.

        This operation retrieves a single variable, given its name and the  management group it was
        created at.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :return: Variable or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.Variable] = kwargs.pop("cls", None)

        _request = build_variables_get_at_management_group_request(
            management_group_id=management_group_id,
            variable_name=variable_name,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("Variable", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list(self, **kwargs: Any) -> AsyncIterable["_models.Variable"]:
        """Retrieves all variables that are at this subscription level.

        This operation retrieves the list of all variables associated with the given subscription.

        :return: An iterator like instance of either Variable or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.VariableListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_variables_list_request(
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("VariableListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def list_for_management_group(self, management_group_id: str, **kwargs: Any) -> AsyncIterable["_models.Variable"]:
        """Retrieves all variables that are at this management group level.

        This operation retrieves the list of all variables applicable to the management group.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :return: An iterator like instance of either Variable or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.resource.policy.v2022_08_01_preview.models.Variable]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.VariableListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_variables_list_for_management_group_request(
                    management_group_id=management_group_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("VariableListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)


class VariableValuesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.resource.policy.v2022_08_01_preview.aio.PolicyClient`'s
        :attr:`variable_values` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")
        self._api_version = input_args.pop(0) if input_args else kwargs.pop("api_version")

    @distributed_trace_async
    async def delete(  # pylint: disable=inconsistent-return-statements
        self, variable_name: str, variable_value_name: str, **kwargs: Any
    ) -> None:
        """Deletes a variable value.

        This operation deletes a variable value, given its name, the subscription it was created in,
        and the variable it belongs to. The scope of a variable value is the part of its ID preceding
        '/providers/Microsoft.Authorization/variables/{variableName}'.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_variable_values_delete_request(
            variable_name=variable_name,
            variable_value_name=variable_value_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    async def create_or_update(
        self,
        variable_name: str,
        variable_value_name: str,
        parameters: _models.VariableValue,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.VariableValue:
        """Creates or updates a variable value.

        This operation creates or updates a variable value with the given subscription and name for a
        given variable. Variable values are scoped to the variable for which they are created for.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :param parameters: Parameters for the variable value. Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update(
        self,
        variable_name: str,
        variable_value_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.VariableValue:
        """Creates or updates a variable value.

        This operation creates or updates a variable value with the given subscription and name for a
        given variable. Variable values are scoped to the variable for which they are created for.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :param parameters: Parameters for the variable value. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update(
        self,
        variable_name: str,
        variable_value_name: str,
        parameters: Union[_models.VariableValue, IO[bytes]],
        **kwargs: Any
    ) -> _models.VariableValue:
        """Creates or updates a variable value.

        This operation creates or updates a variable value with the given subscription and name for a
        given variable. Variable values are scoped to the variable for which they are created for.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :param parameters: Parameters for the variable value. Is either a VariableValue type or a
         IO[bytes] type. Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue or
         IO[bytes]
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.VariableValue] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "VariableValue")

        _request = build_variable_values_create_or_update_request(
            variable_name=variable_name,
            variable_value_name=variable_value_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("VariableValue", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def get(self, variable_name: str, variable_value_name: str, **kwargs: Any) -> _models.VariableValue:
        """Retrieves a variable value.

        This operation retrieves a single variable value; given its name, subscription it was created
        at and the variable it's created for.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.VariableValue] = kwargs.pop("cls", None)

        _request = build_variable_values_get_request(
            variable_name=variable_name,
            variable_value_name=variable_value_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("VariableValue", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list(self, variable_name: str, **kwargs: Any) -> AsyncIterable["_models.VariableValue"]:
        """List variable values for a variable.

        This operation retrieves the list of all variable values associated with the given variable
        that is at a subscription level.

        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :return: An iterator like instance of either VariableValue or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.VariableValueListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_variable_values_list_request(
                    variable_name=variable_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("VariableValueListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace
    def list_for_management_group(
        self, management_group_id: str, variable_name: str, **kwargs: Any
    ) -> AsyncIterable["_models.VariableValue"]:
        """List variable values at management group level.

        This operation retrieves the list of all variable values applicable the variable indicated at
        the management group scope.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :return: An iterator like instance of either VariableValue or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.VariableValueListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_variable_values_list_for_management_group_request(
                    management_group_id=management_group_id,
                    variable_name=variable_name,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("VariableValueListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def delete_at_management_group(  # pylint: disable=inconsistent-return-statements
        self, management_group_id: str, variable_name: str, variable_value_name: str, **kwargs: Any
    ) -> None:
        """Deletes a variable value.

        This operation deletes a variable value, given its name, the management group it was created
        in, and the variable it belongs to. The scope of a variable value is the part of its ID
        preceding '/providers/Microsoft.Authorization/variables/{variableName}'.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_variable_values_delete_at_management_group_request(
            management_group_id=management_group_id,
            variable_name=variable_name,
            variable_value_name=variable_value_name,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    async def create_or_update_at_management_group(
        self,
        management_group_id: str,
        variable_name: str,
        variable_value_name: str,
        parameters: _models.VariableValue,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.VariableValue:
        """Creates or updates a variable value.

        This operation creates or updates a variable value with the given management group and name for
        a given variable. Variable values are scoped to the variable for which they are created for.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :param parameters: Parameters for the variable value. Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update_at_management_group(
        self,
        management_group_id: str,
        variable_name: str,
        variable_value_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.VariableValue:
        """Creates or updates a variable value.

        This operation creates or updates a variable value with the given management group and name for
        a given variable. Variable values are scoped to the variable for which they are created for.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :param parameters: Parameters for the variable value. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update_at_management_group(
        self,
        management_group_id: str,
        variable_name: str,
        variable_value_name: str,
        parameters: Union[_models.VariableValue, IO[bytes]],
        **kwargs: Any
    ) -> _models.VariableValue:
        """Creates or updates a variable value.

        This operation creates or updates a variable value with the given management group and name for
        a given variable. Variable values are scoped to the variable for which they are created for.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :param parameters: Parameters for the variable value. Is either a VariableValue type or a
         IO[bytes] type. Required.
        :type parameters: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue or
         IO[bytes]
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.VariableValue] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "VariableValue")

        _request = build_variable_values_create_or_update_at_management_group_request(
            management_group_id=management_group_id,
            variable_name=variable_name,
            variable_value_name=variable_value_name,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("VariableValue", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def get_at_management_group(
        self, management_group_id: str, variable_name: str, variable_value_name: str, **kwargs: Any
    ) -> _models.VariableValue:
        """Retrieves a variable value.

        This operation retrieves a single variable value; given its name,  management group it was
        created at and the variable it's created for.

        :param management_group_id: The ID of the management group. Required.
        :type management_group_id: str
        :param variable_name: The name of the variable to operate on. Required.
        :type variable_name: str
        :param variable_value_name: The name of the variable value to operate on. Required.
        :type variable_value_name: str
        :return: VariableValue or the result of cls(response)
        :rtype: ~azure.mgmt.resource.policy.v2022_08_01_preview.models.VariableValue
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop(
            "api_version", _params.pop("api-version", self._api_version or "2022-08-01-preview")
        )
        cls: ClsType[_models.VariableValue] = kwargs.pop("cls", None)

        _request = build_variable_values_get_at_management_group_request(
            management_group_id=management_group_id,
            variable_name=variable_name,
            variable_value_name=variable_value_name,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("VariableValue", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore
